/* anypow v1.0 - small program calculate power of numbers at any size
 *
 * Copyright (C) 2004 Giuseppe Caulo <giusc@softzone.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

typedef struct Node_s {
    int data;
	struct Node_s *next, *prev;
} Node;

typedef struct NodeList_s {
	struct Node_s *head, *bottom;
} NodeList;

Node *newNode(int);


void fatal(const char *fmt, ...)
{
    va_list vargs;

    va_start(vargs, fmt);
    vprintf(fmt, vargs);
    va_end(vargs);
    exit(-1);
}

void initNodeList(NodeList *root)
{
	memset(root, 0x0, sizeof(NodeList));
}

void addNodeListNode(NodeList *root, Node *n)
{
	if (!n || !root) return;
	if (!root->head) {
	  n->prev = NULL;
	  n->next = NULL;
	  root->head = n;
	  root->bottom = n;
	} else {
	  n->prev = root->bottom;
	  n->next = NULL;
	  root->bottom->next = n;
	  root->bottom = n;
    }
}

void delNodeListNode(NodeList *root, Node *n)
{
	Node *p;
	if (!n || !root) return;
	if (n == root->head) {
		n->next->prev = NULL;
		root->head = n->next;
	} else if (n == root->bottom) {
		n->prev->next = NULL;
		root->bottom = n->prev;
	} else {
	  for (p = root->head; p; p = p->next) {
	    if(p == n)
			p->next = n->next;
	  }
    }
    free(n);  /* Safe. */
}

void clearNodeList(NodeList *root)
{
	Node *p;
	if (!root || !root->head) return;
	for (p = root->bottom; p; p = p->prev)
	  if (p->next) free(p->next);
	free(root->head);
	initNodeList(root);
}

int getNodeListCount(NodeList *root)
{
 int count = 0;
 Node *p;

 for(p = root->head; p; p = p->next)
   count++;
 return count;
}


Node *newNode(int item)
{
	Node *np;
	np = (Node *)malloc(sizeof(Node));
	if (!np) {
		fatal("Memory exahusted\n"); /* Wtf are you calculating? */
	}
	memset(np, 0x0, sizeof(Node));
	np->data = item;
	return np;
}


int main(int argc, char* argv[])
{
 int remain;
 int carry;
 int result;
 int i, n, N, R, num;
 NodeList factList;
 Node *p;

 printf("\nAnyPow v1.0 - Calculate power of a number at any size\n" );
 printf("Copyright (C) 2004-2005 Giuseppe Caulo\n");
 printf("Softzone - www.softzone.it\n\n");

 initNodeList(&factList);

 if (argc == 3) {
	 N = strtoul(argv[1], NULL, 10);
	 R = strtoul(argv[2], NULL, 10);
 } else {
   printf("Inserisci il numero:");
   fscanf(stdin,"%d", &N);
   printf("Inserisci la potenza:");
   fscanf(stdin,"%d", &R);
 }
 addNodeListNode(&factList, newNode(N));
 for(n = 2; n <= R; n++)
 {
   remain = carry = 0;

   /* Processa gli elementi uno ad uno fino alla fine della lista */
   for (p = factList.head; p->next != NULL; p = p->next) {
     result = p->data * N + carry;
     if(result >= 10){
       remain = result % 10;
       carry = result / 10;
       p->data = remain;
     }
     else{
       p->data = result;
     }
     carry = result / 10;
   }

   result = p->data * N + carry;

   while(result >= 10){
	 p->data = result % 10;  /* resto */
	 result = result / 10;
	 addNodeListNode(&factList, newNode(0));
     p = p->next;
   }

   p->data = result;

 }

 p = factList.bottom;

 for(p = factList.bottom; p->prev != NULL ; p = p->prev){
    printf("%d", p->data);
 }

 printf("%d\n",p->data);
 num = getNodeListCount(&factList);
 if(num >=5){
   p = factList.bottom;

   printf("\noppure\n\n");

   printf("%d.", p->data);
   p = p->prev;

   for(i=1; i<5; i++) {
     printf("%d", p->data);
     p = p->prev;
   }

  printf("E%d\n", num-1);
 }

 /* Pulisce la memoria */
 clearNodeList(&factList);

 return 0;
}

